@@ -1,10 +1,14 @@
 from __future__ import annotations
 
+import abc
 import ast
 import collections
 import collections.abc
+import re
 from ast import AST
-from typing import Any, Callable, NamedTuple, Sequence, Union
+from typing import Any, Callable, ClassVar, MutableSequence, NamedTuple, Sequence, Union
+
+import astor
 
 Node = Union[AST, Sequence[AST]]
 
@@ -14,6 +18,14 @@ class Replacement(NamedTuple):
     new: str
 
 
+class ASTReplacement(Replacement):
+    pass
+
+
+class StringReplacement(Replacement):
+    pass
+
+
 def _is_iterable(value: Any) -> bool:  # type: ignore[misc]
     """Return whether `value` is a non-string iterable."""
     return not isinstance(value, str) and isinstance(value, collections.abc.Iterable)
@@ -52,18 +64,14 @@ def matches(value: Node, pattern: Node) -> bool:
         pattern, AST
     ), f"pattern is not an AST node: {type(pattern).__name__}"
 
-    fields = [
-        (field, getattr(pattern, field))
+    return all(
+        matches(getattr(value, field), getattr(pattern, field))
         for field in pattern._fields
         if hasattr(pattern, field)
-    ]
-    for field_name, field_value in fields:
-        if not matches(getattr(value, field_name), field_value):
-            return False
-    return True
+    )
 
 
-class Rewriter:
+class ASTRewriter:
     """AST pattern matching to enable rewrite rules."""
 
     def __init__(self) -> None:
@@ -85,7 +93,7 @@ class Rewriter:
         return node
 
 
-def build_import_rewrite(proto: str, dep: str) -> Replacement:
+def build_rewrites(proto: str, dep: str) -> Sequence[Replacement]:
     """Construct a replacement import for `dep`.
 
     Parameters
@@ -119,30 +127,106 @@ def build_import_rewrite(proto: str, dep: str) -> Replacement:
         old = f"from {from_} import {part}_pb2 as {as_}"
         new = f"from {leading_dots}{from_} import {part}_pb2 as {as_}"
 
-    return Replacement(old=old, new=new)
+    return [
+        ASTReplacement(old=old, new=new),
+        StringReplacement(
+            old=f"import {'.'.join(parts)}_pb2",
+            new=(
+                f"from {leading_dots or '.'} import {parts[0]}"
+                + "_pb2" * (not import_parts)
+            ),
+        ),
+    ]
 
 
-class ImportRewriter(ast.NodeTransformer):
+class BaseRewriter(abc.ABC):
+    replacement_type: ClassVar[type[Replacement]]
+
+    @abc.abstractmethod
+    def rewrite(self, src: str) -> str:
+        ...
+
+    @abc.abstractmethod
+    def do_register_rewrite(self, replacement: Replacement) -> None:
+        ...
+
+    def register_rewrite(self, replacement: Replacement) -> None:
+        """Register a rewrite rule for turning `old` into `new`."""
+        if isinstance(replacement, self.__class__.replacement_type):
+            self.do_register_rewrite(replacement)
+
+
+class ImportNodeTransformer(ast.NodeTransformer):
     """A NodeTransformer to apply rewrite rules."""
 
-    def __init__(self) -> None:
-        self.rewrite = Rewriter()
-
-    def register_import_rewrite(self, replacement: Replacement) -> None:
-        """Register a rewrite rule for turning `old` into `new`."""
-        (old_import,) = ast.parse(replacement.old).body
-        (new_import,) = ast.parse(replacement.new).body
-
-        def _rewrite(_: AST, new_import: AST = new_import) -> AST:
-            return new_import
-
-        if all(not matches(old_import, pat) for pat, _ in self.rewrite.funcs):
-            self.rewrite.register(old_import)(_rewrite)
-
-        assert sum(matches(old_import, pat) for pat, _ in self.rewrite.funcs) == 1
+    def __init__(self, ast_rewriter: ASTRewriter) -> None:
+        self.ast_rewriter = ast_rewriter
 
     def visit_Import(self, node: ast.Import) -> AST:
-        return self.rewrite(node)
+        return self.ast_rewriter(node)
 
     def visit_ImportFrom(self, node: ast.ImportFrom) -> AST:
-        return self.rewrite(node)
+        return self.ast_rewriter(node)
+
+
+class ASTImportRewriter(BaseRewriter):
+    replacement_type: ClassVar[type[Replacement]] = ASTReplacement
+
+    def __init__(self) -> None:
+        self.node_transformer = ImportNodeTransformer(ASTRewriter())
+
+    def do_register_rewrite(self, replacement: Replacement) -> None:
+        """Register a rewrite rule for turning `old` into `new`."""
+        (old_node,) = ast.parse(replacement.old).body
+        (new_node,) = ast.parse(replacement.new).body
+
+        def _rewrite(_: AST, repl: AST = new_node) -> AST:
+            return repl
+
+        funcs = self.node_transformer.ast_rewriter.funcs
+        if all(not matches(old_node, pat) for pat, _ in funcs):
+            self.node_transformer.ast_rewriter.register(old_node)(_rewrite)
+        assert (
+            sum(matches(old_node, pat) for pat, _ in funcs) == 1
+        ), f"more than one rewrite rule found for pattern `{replacement.old}`"
+
+    def rewrite(self, src: str) -> str:
+        return astor.to_source(self.node_transformer.visit(ast.parse(src)))
+
+
+class StringReplaceImportRewriter(BaseRewriter):
+    replacement_type: ClassVar[type[Replacement]] = StringReplacement
+
+    def __init__(self) -> None:
+        self.replacements: MutableSequence[tuple[re.Pattern[str], str]] = []
+
+    def do_register_rewrite(self, replacement: Replacement) -> None:
+        """Register a rewrite rule for turning `old` into `new`."""
+        self.replacements.append(
+            (
+                re.compile(f"^{re.escape(replacement.old)}$", flags=re.MULTILINE),
+                replacement.new,
+            )
+        )
+
+    def rewrite(self, src: str) -> str:
+        for pattern, new in self.replacements:
+            if pattern.search(src) is not None:
+                src = pattern.sub(new, src)
+        return src
+
+
+class ChainedImportRewriter(BaseRewriter):
+    replacement_type: ClassVar[type[Replacement]] = Replacement
+
+    def __init__(self, *rewriters: BaseRewriter) -> None:
+        self.rewriters = rewriters
+
+    def do_register_rewrite(self, replacement: Replacement) -> None:
+        for rewriter in self.rewriters:
+            rewriter.register_rewrite(replacement)
+
+    def rewrite(self, src: str) -> str:
+        for rewriter in self.rewriters:
+            src = rewriter.rewrite(src)
+        return src
